/***********************************************************************
 *
 * Author: Cheuksan Edward Wang
 *
 *     Copyright (c) 2003 Cheuksan Edward Wang, All rights reserved
 *     
 *     This program is free software; you can redistribute it and/or 
 *     modify it under the terms of the GNU General Public License as 
 *     published by the Free Software Foundation; either version 2 of 
 *     the License, or (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *     GNU General Public License for more details.
 * 
 *     To receive a copy of the GNU General Public License, please write
 *     to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 *     Boston, MA 02111-1307 USA
 *
 ***********************************************************************/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include <assert.h>
#include <linux/types.h>
#include "thinkpad_common.h"
#include "smapidev.h"
#include <gconf/gconf-client.h>
#include <fcntl.h>

#include <pcmcia/cs_types.h>
#include <pcmcia/cs.h>
#include <pcmcia/cistpl.h>
#include <pcmcia/ds.h>

extern int intFiledesc;

void
on_quit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  close(intFiledesc);
  gtk_main_quit();
}

void
on_preferences1_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  static GtkWidget *preferences = NULL;

  if (!preferences) {
    preferences = create_dialog10();
    g_object_add_weak_pointer(G_OBJECT(preferences), (void**)&(preferences));
    gtk_widget_show(preferences);
  }
  /* need to mention permissions */
  /* need to move it to front */
}

void
on_info1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  static GtkWidget *info_dialog = NULL;
  
  /* don't show multiple info windows */
  if (!info_dialog) {
    info_dialog = create_dialog1();
    g_object_add_weak_pointer(G_OBJECT(info_dialog), (void**)&(info_dialog));
    gtk_widget_show(info_dialog);
  }
}


void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  static GtkWidget *about = NULL;

  if (!about) {
    about = create_about2();
    g_object_add_weak_pointer(G_OBJECT(about), (void**)&(about));
    gtk_widget_show(about);
  }
  /* need to move it to front */
}

struct {byte rc; char *errstr;} rcenc[]= // see TP 770 TechRef page 87
{
    {0x00,"done."},
    {0x53,"SMAPI fuction is not available"},
    {0x81,"Invalid parameter"},
    {0x86,"Function is not supported"},
    {0x90,"System error"},
    {0x91,"System is invalid"},
    {0x92,"System is busy"},
    {0xa0,"Device error (disk read error)"},
    {0xa1,"Device is busy"},
    {0xa2,"Device is not attached"},
    {0xa3,"Device is disbled"},
    {0xa4,"Request parameter is out of range"},
    {0xa5,"Request parameter is not accepted"},
    {0xff,"Unknown return code!"}
};

#define RCENCSIZE ((sizeof(rcenc)/sizeof(rcenc[1]))-1)

/***********************************************************************
 *
 * rc2errstr()
 *
 ***********************************************************************/

char *
rc2errstr(byte rc)
{
  int i;

  for(i=0;i<RCENCSIZE;i++) if(rc==rcenc[i].rc) return rcenc[i].errstr;

  return rcenc[RCENCSIZE].errstr;
}

static void show_error(int code) {
  GtkWidget *error_dialog;
    error_dialog = create_dialog6();
    gtk_label_set_text(GTK_LABEL(lookup_widget(error_dialog,
					       "error_message")),
		       rc2errstr(code));
    gtk_widget_show(error_dialog);
}

/***********************************************************************
 *
 * do_smapi()
 *
 ***********************************************************************/

smapi_outparm_t *
do_smapi(byte func, byte sub_func, word p1, word p2,
	 word p3, dword p4, dword p5, int error_p)
{
  static smapi_ioparm_t parm;
  char *errstr;

  parm.in.bFunc=func;
  parm.in.bSubFunc=sub_func;
  parm.in.wParm1=p1;
  parm.in.wParm2=p2;
  parm.in.wParm3=p3;
  parm.in.dwParm4=p4;
  parm.in.dwParm5=p5;

  if(ioctl_smapi(intFiledesc,&parm)<0) return NULL;

  if(error_p && (parm.out.bRc != 0)) {
    show_error(parm.out.bRc);
  }

  return &(parm.out);
}

static void set_power(int bitnr, int param, int result_check) {
  smapi_outparm_t *result;
  int tmp=0;

  assert(result=do_smapi(0x22,0x00,0,0,0,0,0,TRUE));
  tmp=(result->wParm2 & (0xff00 >> bitnr)) | (param << bitnr);
  /* change config */
  assert(do_smapi(0x22,0x01,tmp,0,0,0,0,TRUE));
  /* this is just checking */
  /* assert(result=do_smapi(0x22,0x00,0,0,0,0,0,FALSE));
     assert(((result->wParm2>>bitnr) & 0x00ff) == result_check);
  */
}

#define TIMER_MANUAL_AC		0x0100
#define TIMER_MANUAL_BATTERY	0x0200
#define TIMER_HIGH		0xf300
#define TIMER_AUTO		0xf400

static int setting_delay_timer=FALSE;
static int minute_allowed[] = {0, 1, 2, 3, 5, 10, 15, 20, 25, 30, 45, 60};

static int num_to_option(num) {
  int i;
  for (i=0; i<12; i++) {
    if (minute_allowed[i] == num) {
      return i;
    }
  }
  return -1;
}

static void set_delay_timer_config(int timer,
				   char *optionmenu, GtkWidget *widget) {
  smapi_outparm_t *result;
  GtkOptionMenu *option_menu;

  /* set the delay timer initial options */
  assert(result=do_smapi(0x32, 0x02, timer,0,0,0,0,FALSE));
  setting_delay_timer = TRUE;
  option_menu = GTK_OPTION_MENU(lookup_widget(widget, optionmenu));

  if ((result->bRc != 0x81) && result->wParm1 & 0x0100) {
    /* capable */
    if (timer == TIMER_HIGH) {
      /* high mode, changing it gets invalid parameter, tpctl can't change it
       */
      gtk_widget_set_sensitive(GTK_WIDGET(option_menu), FALSE);
    } else {
      gtk_widget_set_sensitive(GTK_WIDGET(option_menu), TRUE);
    }
    /* set delay */
    gtk_option_menu_set_history(option_menu,
				num_to_option(result->wParm2 & 0x00ff));
  } else {
    /* not capable */
    gtk_option_menu_set_history(option_menu, 0);
    gtk_widget_set_sensitive(GTK_WIDGET(option_menu), FALSE);
  }
  setting_delay_timer = FALSE;
}

#define AC_POWER 0

void
on_high_performance1_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_power(AC_POWER, 0x0000, 0);
  set_delay_timer_config(TIMER_HIGH, "optionmenu18", GTK_WIDGET(menuitem));
}

void
on_automatic1_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_power(AC_POWER, 0x0001, 1);
  set_delay_timer_config(TIMER_AUTO, "optionmenu18", GTK_WIDGET(menuitem));
}

void
on_customized1_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_power(AC_POWER, 0x0002, 2);
  set_delay_timer_config(TIMER_MANUAL_AC, "optionmenu18",
			 GTK_WIDGET(menuitem));
}

#define BATT_POWER 8

void
on_batt_high_performance_activate      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_power(BATT_POWER, 0x0000, 0);
  set_delay_timer_config(TIMER_HIGH, "optionmenu17", GTK_WIDGET(menuitem));
}


void
on_batt_automatic_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_power(BATT_POWER, 0x0001, 1);
  set_delay_timer_config(TIMER_AUTO, "optionmenu17", GTK_WIDGET(menuitem));
}

void
on_batt_customized_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_power(BATT_POWER, 0x0002, 2);
  set_delay_timer_config(TIMER_MANUAL_BATTERY, "optionmenu17",
			 GTK_WIDGET(menuitem));
}


void
on_closebutton1_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
}


gboolean
on_app1_delete_event                   (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  close(intFiledesc);
  gtk_main_quit();
  return FALSE;
}

void
on_sensors_map                         (GtkWidget       *widget,
                                        gpointer         user_data)
{
	smapidev_sensorinfo_t sensorinfoThe;
	int intRtn;

	sensorinfoThe.sizeStruct = sizeof( sensorinfoThe );
	intRtn = smapidev_GetSensorInfo( intFiledesc, &sensorinfoThe );
	if ( intRtn ) {
	  /* error */
	  fprintf(stderr, "%d\n", intRtn);
	  return;
	}

	gtk_label_set_text(GTK_LABEL(lookup_widget(widget, "lid_label")),
			   sensorinfoThe.fLidClosed ? "Yes" : "No");
	gtk_label_set_text(GTK_LABEL(lookup_widget(widget, "keyboard_label")),
			   sensorinfoThe.fKeyboardOpen ? "Yes" : "No");
	gtk_label_set_text(GTK_LABEL(lookup_widget(widget, "AC_label")),
			   sensorinfoThe.fACAdapterAttached ? "Yes" : "No");
}

void
on_software_information1_activate      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  static GtkWidget *software_info = NULL;

  if (!software_info) {
    software_info = create_dialog2();
    g_object_add_weak_pointer(G_OBJECT(software_info),
			      (void**)&(software_info));
    gtk_widget_show(software_info);
  }
  /* need to move it to front */
}

void
on_close_software_info_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
}

static void set_event_bits (int func, int nr, int cond) {
  smapi_outparm_t *result;

  assert(result=do_smapi(func,0x00,0,0,0,0,0,TRUE));
  switch(nr)
    {
    case 1: result->wParm1  = cond; break;
    case 2: result->wParm2  = cond; break;
    case 3: result->wParm3  = cond; break;
    case 4: result->dwParm4 = cond; break;
    case 5: result->dwParm5 = cond; break;
    }
  assert(do_smapi(func,0x01,result->wParm2,result->wParm1,
		  0,result->dwParm4, result->dwParm5, TRUE));
}

#define BIT_STANDBY     3
#define BIT_SUSPEND	4
#define BIT_REDISAFE	5
#define BIT_HIBERNATION 6
#define BIT_POWEROFF	7

void
on_nonsticky1_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  assert(do_smapi(0x13,0x03,0x00,0,0,0,0,TRUE));
}

void
on_sticky1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  assert(do_smapi(0x13,0x03,0x01,0,0,0,0,TRUE));
}

void
on_locked1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  assert(do_smapi(0x13,0x03,0x03,0,0,0,0,TRUE));
}

void
on_standby_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
	sync();
	assert(do_smapi(0x70, 0x00, 0, 0, 0, 0, 0, TRUE));
}

/*====================================================================*/

static int lookup_dev(char *name)
{
    FILE *f;
    int n;
    char s[32], t[32];
    
    f = fopen("/proc/devices", "r");
    if (f == NULL)
	return -errno;
    while (fgets(s, 32, f) != NULL) {
	if (sscanf(s, "%d %s", &n, t) == 2)
	    if (strcmp(name, t) == 0)
		break;
    }
    fclose(f);
    if (strcmp(name, t) == 0)
	return n;
    else
	return -ENODEV;
} /* lookup_dev */

/*====================================================================*/

static int open_sock(int sock, int major)
{
    static char *paths[] = {
	"/var/lib/pcmcia", "/var/run", "/dev", "/tmp", NULL
    };
    int fd;
    char **p, fn[64];
    dev_t dev = (major<<8) + sock;

    for (p = paths; *p; p++) {
	sprintf(fn, "%s/cc-%d", *p, getpid());
	if (mknod(fn, (S_IFCHR|S_IREAD|S_IWRITE), dev) == 0) {
	    fd = open(fn, O_RDONLY);
	    unlink(fn);
	    if (fd >= 0)
		return fd;
	    if (errno == ENODEV)
		break;
	}
    }
    return -1;
} /* open_sock */

/*====================================================================*/

#define MAX_SOCKS 8

static void cardctl_eject() {
    int fd[MAX_SOCKS], ns, ret, major;

    major = lookup_dev("pcmcia");
    if (major < 0) {
      if (major != -ENODEV) {
	/* ENODEV means no pcmcia driver in /proc/devices */
	perror("could not open /proc/devices");
      }
      return;
    }

    for (ns = 0; ns < MAX_SOCKS; ns++) {
      fd[ns] = open_sock(ns, major);
      if (fd[ns] < 0) break;
      ret = ioctl(fd[ns], DS_EJECT_CARD);
      if ((ret != 0) && (errno != ENODEV)) {
	perror("ioctl()");
      }
    }
}

static gchar *eject_pcmcia_before_suspend = "/apps/configure-thinkpad/eject_pcmcia_before_suspend";
static gchar *eject_pcmcia_before_hibernate = "/apps/configure-thinkpad/eject_pcmcia_before_hibernate";

void
on_suspend_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  smapi_outparm_t *result;
  GConfClient *client;

  sync();

  client = gconf_client_get_default();
  if (gconf_client_get_bool(client, eject_pcmcia_before_suspend, NULL)) {
    cardctl_eject();
  }

  assert(result=do_smapi(0x70, 0x01, 0, 0, 0, 0, 0, TRUE));
  if (result->bRc != 0) {
    show_error(result->bRc);
  }
}

void
on_hibernate_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  smapi_outparm_t *result;
  GConfClient *client;

  sync();
  client = gconf_client_get_default();
  if (gconf_client_get_bool(client, eject_pcmcia_before_hibernate, NULL)) {
    cardctl_eject();
  }

  assert(result=do_smapi(0x70, 0x02, 0, 0, 0, 0, 0, FALSE));
  if (result->bRc == 0x91) {
    /* System is invalid */
    gtk_widget_show(create_dialog3());
    /* check for connected ac adaptor and no preference set */
  } else if (result->bRc != 0) {
    show_error(result->bRc);
  }
}

static int to_timer(int power_mode, int battery_p) {
    switch (power_mode) {
    case 0: return(TIMER_HIGH);
    case 1: return(TIMER_AUTO);
    case 2: if (battery_p) {
      return(TIMER_MANUAL_BATTERY);
      } else {
	return(TIMER_MANUAL_AC);
      }
    }
}

void
on_ac_power_mode_realize               (GtkWidget       *widget,
                                        gpointer         user_data)
{
  /* get the currect ac power mode */
  smapi_outparm_t *result;
  int tmp;

  assert(result=do_smapi(0x22,0x00,0,0,0,0,0,TRUE));
  tmp = (result->wParm2>>AC_POWER) & 0x00ff;
  /* assert(tmp>=0);
     assert(tmp<=2); */
  gtk_option_menu_set_history(GTK_OPTION_MENU(widget), tmp);

  set_delay_timer_config(to_timer(tmp, FALSE), "optionmenu18", widget);
}

void
on_batt_power_mode_realize             (GtkWidget       *widget,
                                        gpointer         user_data)
{
  smapi_outparm_t *result;
  int tmp;

  assert(result=do_smapi(0x22,0x00,0,0,0,0,0,TRUE));
  tmp = (result->wParm2>>BATT_POWER) & 0x00ff;
  /* assert(tmp>=0);
     assert(tmp<=2); */
  gtk_option_menu_set_history(GTK_OPTION_MENU(widget), tmp);
  set_delay_timer_config(to_timer(tmp, TRUE), "optionmenu17", widget);
}

void
on_redisafe_realize                    (GtkWidget       *widget,
                                        gpointer         user_data)
{
  smapi_outparm_t *result;
  assert(result=do_smapi(0x30,0x00,0,0,0,0,0,TRUE));
  if ((result->wParm2 & 0x0100) == 0) {
    /* redisafe not capable */
    gtk_widget_set_sensitive(widget, 0);
  } else {
    /* make the dialog consistent with the current setting */
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget),
				 result->wParm2 & 0x0001);
  }
}

void
on_redisafe_toggled                    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  smapi_outparm_t *result;
  /* what does this do? */
  if (gtk_toggle_button_get_active(togglebutton)) {
    assert(do_smapi(0x30,0x01,0x0001,0,0,0,0,TRUE));
    /* check */
    /* assert(result=do_smapi(0x30,0x00,0,0,0,0,0,TRUE));
       assert(result->wParm2 & 0x0001);
    */
  } else {
    assert(do_smapi(0x30,0x01,0x0000,0,0,0,0,TRUE));
    /* check */
    /* assert(result=do_smapi(0x30,0x00,0,0,0,0,0,TRUE));
       assert(!(result->wParm2 & 0x0001));
    */
  }
  /* assert(result->wParm2 & 0x0100); */
}

void
on_fn_hotkey_realize                   (GtkWidget       *widget,
                                        gpointer         user_data)
{
  smapi_outparm_t *result;
  assert(result=do_smapi(0x13,0x02,0,0,0,0,0,TRUE));
  switch(result->wParm2 & 0x0300)
    {
    case 0x0100: 
      /* disable lock */
      gtk_widget_set_sensitive(lookup_widget(widget, "locked1"), 0);
      break;
    case 0x0200:
      /* disable sticky */
      gtk_widget_set_sensitive(lookup_widget(widget, "sticky1"), 0);
      break;
    case 0x0300:
      break;
    case 0x0000:
      gtk_widget_set_sensitive(lookup_widget(widget, "locked1"), 0);
      gtk_widget_set_sensitive(lookup_widget(widget, "sticky1"), 0);
      break;
    }
  switch(result->wParm2 & 0x00ff) {
    case 0x00:
      gtk_option_menu_set_history(GTK_OPTION_MENU(widget), 0);
      break;
    case 0x01:
      gtk_option_menu_set_history(GTK_OPTION_MENU(widget), 1);
      break;
    case 0x03:
      gtk_option_menu_set_history(GTK_OPTION_MENU(widget), 2);
      break;
  default:
    fprintf(stderr, "unknown\n");
  }
}

static void resume_realize (GtkWidget *widget, int bitnr) {
  smapi_outparm_t *result;
  int tmp=0;

  assert(result=do_smapi(0x34,0x00,0,0,0,0,0,TRUE));
  tmp = ((((result->wParm3 & 0x0f) << 8) | 
	  (result->wParm2 & 0x0f)) >> bitnr) & 0x0101;
  if (tmp == 0x00 || tmp == 0x0001) {
    /* not capable */
    gtk_widget_set_sensitive(widget, 0);
  } else if (tmp == 0x0101) {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget),
				 1);
  }
}

static void toggle_resume (int bitnr, GtkToggleButton *togglebutton) {
  smapi_outparm_t *result;
  int tmp;

  assert(result=do_smapi(0x34,0x00,0,0,0,0,0,TRUE));
  if (gtk_toggle_button_get_active(togglebutton)) {
    tmp=result->wParm2 | (0x0001 << bitnr);
  } else {
    tmp=result->wParm2 & ~(0x0001 << bitnr);
  }
  assert(do_smapi(0x34,0x01,tmp,0,0,0,0,TRUE));
  /* check */
  /*
  assert(result=do_smapi(0x34,0x00,0,0,0,0,0,TRUE));
  if (gtk_toggle_button_get_active(togglebutton)) {
    assert ((((((result->wParm3 & 0x0f) << 8) | 
	       (result->wParm2 & 0x0f)) >> bitnr) & 0x0101) == 0x101);
  }
  */   
}
  
void
on_opening_lid_realize                 (GtkWidget       *widget,
                                        gpointer         user_data)
{
  resume_realize(widget, 1);
}

void
on_opening_lid_toggled                 (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  /* bug: can't disable it, keep returning 0x101 */
  /* doesn't work for ntpctl or tpctl */
  toggle_resume(1, togglebutton);
}

void
on_appointment_resume_realize          (GtkWidget       *widget,
                                        gpointer         user_data)
{
  resume_realize(widget, 2);
}

void
on_appointment_resume_toggled          (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  toggle_resume(2, togglebutton);
  if (gtk_toggle_button_get_active(togglebutton)) {
    gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(togglebutton),
					   "frame6"), 1);
  } else {
    gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(togglebutton),
					   "frame6"), 0);
  }
}

void
on_hardware_resume_realize             (GtkWidget       *widget,
                                        gpointer         user_data)
{
  resume_realize(widget, 0);
}

void
on_hardware_resume_toggled             (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  /* this also doesn't work */
  /* what does this mean? */
  toggle_resume(0, togglebutton);
}

void
on_serial_ri_resume_realize            (GtkWidget       *widget,
                                        gpointer         user_data)
{
  /* what is this? */
  resume_realize(widget, 3);
}

void
on_serial_ri_resume_toggled            (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  toggle_resume(3, togglebutton);
}

#define REQTYPE_CMOS             0x01
#define REQTYPE_current_hardware 0x00

static smapi_outparm_t *handle_internal_pointing_device(int arg, int reqtype) {
  smapi_outparm_t *result;
  int tmp;

  assert(result=do_smapi(0x11,0x02,reqtype << 8,0,0,0,0,TRUE));
  tmp=(result->wParm2 & 0x0200) | arg;
  assert(result = do_smapi(0x11,0x03,tmp |
			   ((reqtype==REQTYPE_CMOS) ? 0x02 : 0x01),
			   0, 0,0,0,FALSE));
  return result;
}

void
on_enable1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  handle_internal_pointing_device(0x0100, REQTYPE_current_hardware);
  /* doesn't work immediately. It works after suspend, resume */
  gtk_widget_show(create_dialog4());
  handle_internal_pointing_device(0x0100, REQTYPE_CMOS);
}

void
on_disable2_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  handle_internal_pointing_device(0x0000, REQTYPE_current_hardware);
  /* this works immediately */
  handle_internal_pointing_device(0x0000, REQTYPE_CMOS);
}

void
on_internal_pointing_device_automatic_activate
                                        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  smapi_outparm_t *result;
  int current_hardware_bRc;
  result = handle_internal_pointing_device(0x0400, REQTYPE_current_hardware);
  current_hardware_bRc = result->bRc;
  /* I got Invalid parameter! */
  handle_internal_pointing_device(0x0400, REQTYPE_CMOS);
  if (current_hardware_bRc == 0x81) {
    /* invalid parameter */
    gtk_widget_show(create_dialog5());
  } else if (current_hardware_bRc != 0) {
    show_error(current_hardware_bRc);
  }
}

void
on_internal_pointing_device_realize    (GtkWidget       *widget,
                                        gpointer         user_data)
{
  smapi_outparm_t *result;
  int reqtype = REQTYPE_current_hardware;

  assert(result=do_smapi(0x11,0x02,reqtype << 8,0,0,0,0,TRUE));
  switch((result->wParm2 >> 8)& 0x05) {
  case 0x00: /* disabled */
    gtk_option_menu_set_history(GTK_OPTION_MENU(widget), 1);
    break;
  case 0x01: /* enabled */
    /* noop */
    /* gtk_option_menu_set_history(GTK_OPTION_MENU(widget), 0); */
    break;
  case 0x04: /* automatic */
    gtk_option_menu_set_history(GTK_OPTION_MENU(widget), 2);
    break;
  }
  if (!(result->wParm2 & 0x0001)) {
    /* It's not controllable */
    gtk_widget_set_sensitive(widget, 0);
  }
}

void
on_external_pointing_device_realize    (GtkWidget       *widget,
                                        gpointer         user_data)
{
  smapi_outparm_t *result;
  int reqtype = REQTYPE_current_hardware;
  int disabled = FALSE;

  assert(result=do_smapi(0x11,0x02,reqtype << 8,0,0,0,0,TRUE));
  if (!(result->wParm2 & 0x0200)) {
    /* disabled */
    gtk_option_menu_set_history(GTK_OPTION_MENU(widget), 1);
    disabled = TRUE;
  }
  if (!(result->wParm2 & 0x0002)) {
    /* not controllable */
    gtk_widget_set_sensitive(widget, 0);
    if (disabled) {
      gtk_widget_set_sensitive(lookup_widget(widget, "label186"), FALSE);
    }
  }
}

static int sedation_realize(int func, int nr, char *menu1, char *menu2,
			     char* menu3, char* menu4, char *menu5,
			     GtkWidget *widget, char *label) {
  smapi_outparm_t *result;
  int tmp, cap, cond, i, some_capability_p=FALSE;

  assert(result=do_smapi(func,0x00,0,0,0,0,0,TRUE));
  switch(nr)
    {
    case 1: tmp=result->wParm1;	          break;
    case 2: tmp=result->wParm2;		  break;
    case 3: tmp=result->wParm3;		  break;
    case 4: tmp=result->dwParm4 & 0xffff; break;
    case 5: tmp=result->dwParm5 & 0xffff; break;
    }
  cap =(tmp>>8) & 0xff;
  if (cap & (0x0001<<3)) {
    some_capability_p = TRUE;
  } else {
    gtk_widget_set_sensitive(lookup_widget(widget, menu1), 0);
  }
  if (cap & (0x0001<<4)) {
    some_capability_p = TRUE;
  } else {
    gtk_widget_set_sensitive(lookup_widget(widget, menu2), 0);
  }
  if (cap & (0x0001<<5)) {
    some_capability_p = TRUE;
  } else {
    gtk_widget_set_sensitive(lookup_widget(widget, menu3), 0);
  }
  if (cap & (0x0001<<6)) {
    some_capability_p = TRUE;
  } else {
    gtk_widget_set_sensitive(lookup_widget(widget, menu4), 0);
  }
  if (cap & (0x0001<<7)) {
    some_capability_p = TRUE;
  } else {
    gtk_widget_set_sensitive(lookup_widget(widget, menu5), 0);
  }
  cond= tmp & 0xff;
  for(i=0;i<5;i++) {
    if (cond & (0x0001<<(i+3))) {
      /* what does it mean when multiple ones are true? e.g. hw_sw_event
	 printf("%d is true\n"); */
      gtk_option_menu_set_history(GTK_OPTION_MENU(widget), i+1);
      break;
    }
  }
  /* disable the whole menu */
  if (!some_capability_p) {
    gtk_widget_set_sensitive(widget, FALSE);
    gtk_widget_set_sensitive(lookup_widget(widget, label), 0);
  }
  return some_capability_p;
}

#define HW_SW_EVENT_FUNC 0x31
#define HW_SW_EVENT_NR 2

/* What does this actually mean? */
void
on_hw_sw_event_sedation_realize        (GtkWidget       *widget,
                                        gpointer         user_data)
{
  sedation_realize(HW_SW_EVENT_FUNC, HW_SW_EVENT_NR, "hw_sw_event_standby",
		   "hw_sw_event_suspend",
		   "hw_sw_event_redisafe", "hw_sw_event_hibernate", 
		   "hw_sw_event_poweroff", widget, "label153");
}

void
on_hw_sw_event_disable_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(HW_SW_EVENT_FUNC, HW_SW_EVENT_NR, 0);
}

void
on_hw_sw_event_standby_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(HW_SW_EVENT_FUNC, HW_SW_EVENT_NR, 1<<BIT_STANDBY);
}

void
on_hw_sw_event_suspend_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(HW_SW_EVENT_FUNC, HW_SW_EVENT_NR, 1<<BIT_SUSPEND);
}

void
on_hw_sw_event_redisafe_activate       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(HW_SW_EVENT_FUNC, HW_SW_EVENT_NR, 1<<BIT_REDISAFE);
}

void
on_hw_sw_event_hibernate_activate      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(HW_SW_EVENT_FUNC, HW_SW_EVENT_NR, 1<<BIT_HIBERNATION);
}

void
on_hw_sw_event_poweroff_activate       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(HW_SW_EVENT_FUNC, HW_SW_EVENT_NR, 1<<BIT_POWEROFF);
}

#define POWER_SWITCH_FUNC 0x31
#define POWER_SWITCH_NR 4

void
on_pm_power_switch_realize             (GtkWidget       *widget,
                                        gpointer         user_data)
{
  sedation_realize(POWER_SWITCH_FUNC, POWER_SWITCH_NR, "power_switch_standby",
		   "power_switch_suspend",
		   "power_switch_redisafe", "power_switch_hibernate", 
		   "power_switch_poweroff", widget, "label154");
}

void
on_power_switch_disable_activate       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(POWER_SWITCH_FUNC, POWER_SWITCH_NR, 0);
}

void
on_power_switch_standby_activate       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(POWER_SWITCH_FUNC, POWER_SWITCH_NR, 1<<BIT_STANDBY);
}

void
on_power_switch_suspend_activate       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(POWER_SWITCH_FUNC, POWER_SWITCH_NR, 1<<BIT_SUSPEND);
}

void
on_power_switch_redisafe_activate      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(POWER_SWITCH_FUNC, POWER_SWITCH_NR, 1<<BIT_REDISAFE);
}

void
on_power_switch_hibernate_activate     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(POWER_SWITCH_FUNC, POWER_SWITCH_NR, 1<<BIT_HIBERNATION);
}

void
on_power_switch_power_off_activate     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(POWER_SWITCH_FUNC, POWER_SWITCH_NR, 1<<BIT_POWEROFF);
}

#define LID_CLOSURE_FUNC 0x31
#define LID_CLOSURE_NR 5

void
on_lid_closure_realize                 (GtkWidget       *widget,
                                        gpointer         user_data)
{
  sedation_realize(LID_CLOSURE_FUNC, LID_CLOSURE_NR, "lid_closure_standby",
		   "lid_closure_suspend",
		   "lid_closure_redisafe", "lid_closure_hibernate", 
		   "lid_closure_poweroff", widget, "label155");
}

void
on_lid_closure_disable_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(LID_CLOSURE_FUNC, LID_CLOSURE_NR, 0);
}

void
on_lid_closure_standby_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(LID_CLOSURE_FUNC, LID_CLOSURE_NR, 1<<BIT_STANDBY);
}

void
on_lid_closure_suspend_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(LID_CLOSURE_FUNC, LID_CLOSURE_NR, 1<<BIT_SUSPEND);
}

void
on_lid_closure_redisafe_activate       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(LID_CLOSURE_FUNC, LID_CLOSURE_NR, 1<<BIT_REDISAFE);
}

void
on_lid_closure_hibernate_activate      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(LID_CLOSURE_FUNC, LID_CLOSURE_NR, 1<<BIT_HIBERNATION);
}

void
on_lid_closure_power_off_activate      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(LID_CLOSURE_FUNC, LID_CLOSURE_NR, 1<<BIT_POWEROFF);
}

#define STANDBY_TIMER_FUNC 0x32
#define STANDBY_TIMER_NR 4

void
on_standby_timer_realize               (GtkWidget       *widget,
                                        gpointer         user_data)
{
  sedation_realize(STANDBY_TIMER_FUNC, STANDBY_TIMER_NR,
		   "standby_timer_standby", "standby_timer_suspend",
		   "standby_timer_redisafe", "standby_timer_hibernate", 
		   "standby_timer_poweroff", widget, "label156");
}

void
on_standby_timer_disable_activate      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(STANDBY_TIMER_FUNC, STANDBY_TIMER_NR, 0);
}


void
on_standby_timer_standby_activate      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(STANDBY_TIMER_FUNC, STANDBY_TIMER_NR, 1<<BIT_STANDBY);
}


void
on_standby_timer_suspend_activate      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(STANDBY_TIMER_FUNC, STANDBY_TIMER_NR, 1<<BIT_SUSPEND);
}


void
on_standby_timer_redisafe_activate     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(STANDBY_TIMER_FUNC, STANDBY_TIMER_NR, 1<<BIT_REDISAFE);
}


void
on_standby_timer_hibernate_activate    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(STANDBY_TIMER_FUNC, STANDBY_TIMER_NR, 1<<BIT_HIBERNATION);
}


void
on_standby_timer_power_off_activate    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(STANDBY_TIMER_FUNC, STANDBY_TIMER_NR, 1<<BIT_POWEROFF);
}

#define SYSTEM_TIMER_FUNC 0x32
#define SYSTEM_TIMER_NR 2

void
on_system_timer_realize                (GtkWidget       *widget,
                                        gpointer         user_data)
{
  sedation_realize(SYSTEM_TIMER_FUNC, SYSTEM_TIMER_NR,
		   "system_timer_standby", "system_timer_suspend",
		   "system_timer_redisafe", "system_timer_hibernate", 
		   "system_timer_poweroff", widget, "label157");
}

void
on_system_timer_disable_activate       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(SYSTEM_TIMER_FUNC, SYSTEM_TIMER_NR, 0);
}


void
on_system_timer_standby_activate       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(SYSTEM_TIMER_FUNC, SYSTEM_TIMER_NR, 1<<BIT_STANDBY);
}


void
on_system_timer_suspend_activate       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(SYSTEM_TIMER_FUNC, SYSTEM_TIMER_NR, 1<<BIT_SUSPEND);
}


void
on_system_timer_redisafe_activate      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(SYSTEM_TIMER_FUNC, SYSTEM_TIMER_NR, 1<<BIT_REDISAFE);
}


void
on_system_timer_hibernate_activate     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(SYSTEM_TIMER_FUNC, SYSTEM_TIMER_NR, 1<<BIT_HIBERNATION);
}


void
on_system_timer_power_off_activate     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(SYSTEM_TIMER_FUNC, SYSTEM_TIMER_NR, 1<<BIT_POWEROFF);
}

#define H_FROM_S_TIMER_FUNC 0x32
#define H_FROM_S_TIMER_NR 5

void
on_h_from_s_timer_realize              (GtkWidget       *widget,
                                        gpointer         user_data)
{
  int some_capability_p;
  some_capability_p =
    sedation_realize(H_FROM_S_TIMER_FUNC, H_FROM_S_TIMER_NR,
		   "h_from_s_timer_standby", "h_from_s_timer_suspend",
		   "h_from_s_timer_redisafe", "h_from_s_timer_hibernate", 
		   "h_from_s_timer_poweroff", widget, "label158");
  if (!some_capability_p) {
    gtk_widget_set_sensitive(lookup_widget(widget, "hbox17"), 0);
  }
}


void
on_hs_timer_disable_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(H_FROM_S_TIMER_FUNC, H_FROM_S_TIMER_NR, 0);
}


void
on_hs_timer_standby_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(H_FROM_S_TIMER_FUNC, H_FROM_S_TIMER_NR, 1<<BIT_STANDBY);
}


void
on_hs_timer_suspend_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(H_FROM_S_TIMER_FUNC, H_FROM_S_TIMER_NR, 1<<BIT_SUSPEND);
}


void
on_hs_timer_redisafe_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(H_FROM_S_TIMER_FUNC, H_FROM_S_TIMER_NR, 1<<BIT_REDISAFE);
}


void
on_hs_timer_hibernate_activate         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(H_FROM_S_TIMER_FUNC, H_FROM_S_TIMER_NR, 1<<BIT_HIBERNATION);
}


void
on_hs_timer_power_off_activate         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(H_FROM_S_TIMER_FUNC, H_FROM_S_TIMER_NR, 1<<BIT_POWEROFF);
}

#define LOW_BATTERY_FUNC 0x33
#define LOW_BATTERY_NR 2

void
on_low_battery_menu_realize            (GtkWidget       *widget,
                                        gpointer         user_data)
{
  sedation_realize(LOW_BATTERY_FUNC, LOW_BATTERY_NR, "low_battery_standby",
		   "low_battery_suspend",
		   "low_battery_redisafe", "low_battery_hibernate", 
		   "low_battery_poweroff", widget, "label159");
}

void
on_low_battery_disable_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(LOW_BATTERY_FUNC, LOW_BATTERY_NR, 0);
}

void
on_low_battery_standby_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(LOW_BATTERY_FUNC, LOW_BATTERY_NR, 1<<BIT_STANDBY);
}


void
on_low_battery_suspend_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(LOW_BATTERY_FUNC, LOW_BATTERY_NR, 1<<BIT_SUSPEND);
}


void
on_low_battery_redisafe_activate       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(LOW_BATTERY_FUNC, LOW_BATTERY_NR, 1<<BIT_REDISAFE);
}


void
on_low_battery_hibernate_activate      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(LOW_BATTERY_FUNC, LOW_BATTERY_NR, 1<<BIT_HIBERNATION);
}


void
on_low_battery_power_off_activate      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(LOW_BATTERY_FUNC, LOW_BATTERY_NR, 1<<BIT_POWEROFF);
}

#define ENV_EXHAUSTED_FUNC 0x32
#define ENV_EXHAUSTED_NR 5

void
on_env_exhausted_realize               (GtkWidget       *widget,
                                        gpointer         user_data)
{
  sedation_realize(ENV_EXHAUSTED_FUNC, ENV_EXHAUSTED_NR,
		   "env_exhausted_standby",
		   "env_exhausted_suspend",
		   "env_exhausted_redisafe", "env_exhausted_hibernate", 
		   "env_exhausted_poweroff", widget, "label160");
}


void
on_env_exhausted_disable_activate      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(ENV_EXHAUSTED_FUNC, ENV_EXHAUSTED_NR, 0);
}


void
on_env_exhausted_standby_activate      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(ENV_EXHAUSTED_FUNC, ENV_EXHAUSTED_NR, 1<<BIT_STANDBY);
}


void
on_env_exhausted_suspend_activate      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(ENV_EXHAUSTED_FUNC, ENV_EXHAUSTED_NR, 1<<BIT_SUSPEND);
}


void
on_env_exhausted_redisafe_activate     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(ENV_EXHAUSTED_FUNC, ENV_EXHAUSTED_NR, 1<<BIT_REDISAFE);
}


void
on_env_exhausted_hibernate_activate    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(ENV_EXHAUSTED_FUNC, ENV_EXHAUSTED_NR, 1<<BIT_HIBERNATION);
}


void
on_env_exhausted_power_off_activate    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  set_event_bits(ENV_EXHAUSTED_FUNC, ENV_EXHAUSTED_NR, 1<<BIT_POWEROFF);
}


void
on_hib_button_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
}


void
on_notification_closebutton_clicked    (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
}


void
on_closebutton4_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
}

static void handle_external_pd(int enable_p, int reqtype) {
  smapi_outparm_t *result;
  int tmp;
  assert(result=do_smapi(0x11,0x02,reqtype << 8,0,0,0,0,TRUE));
  if (enable_p) {
    tmp=(result->wParm2 | 0x0200) & 0x0700;
  } else {
    tmp=result->wParm2 & 0x0500;
  }
  assert(do_smapi(0x11,0x03,tmp | ((reqtype==REQTYPE_CMOS) ? 0x02 : 0x01),
		  0,0,0,0,TRUE));
}

void
on_ext_pd_enable_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  handle_external_pd(TRUE, REQTYPE_CMOS);
  handle_external_pd(TRUE, REQTYPE_current_hardware);
}

void
on_ext_pd_disable_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  handle_external_pd(FALSE, REQTYPE_CMOS);
  handle_external_pd(FALSE, REQTYPE_current_hardware);
}


void
on_closebutton5_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
}

static void appointment_changed(GtkWidget *widget) {
  time_t time;
  struct tm *time_struct;
  int parm4, parm5;

  time = gnome_date_edit_get_time(GNOME_DATE_EDIT(lookup_widget(widget,
								"dateedit1")));
  time_struct = localtime(&time);
  parm4 = (bcd8_of_byte(time_struct->tm_hour)) << 16 |
    (bcd8_of_byte(time_struct->tm_min)) << 8 |
    (bcd8_of_byte(time_struct->tm_sec));
  parm5 = (bcd8_of_byte(time_struct->tm_year - 100)) << 16 |
    (bcd8_of_byte(time_struct->tm_mon + 1)) << 8 |
    (bcd8_of_byte(time_struct->tm_mday));
  if (gtk_toggle_button_get_active
      (GTK_TOGGLE_BUTTON(lookup_widget(widget, "checkbutton10")))) {
    parm5 |= 0x80000000;
  }
  assert(do_smapi(0x34,0x03,0,0,0,parm4,parm5,TRUE));
}
  
void
on_dateedit1_date_changed              (GnomeDateEdit   *gnomedateedit,
                                        gpointer         user_data)
{
  appointment_changed(GTK_WIDGET(gnomedateedit));
}


void
on_dateedit1_time_changed              (GnomeDateEdit   *gnomedateedit,
                                        gpointer         user_data)
{
  appointment_changed(GTK_WIDGET(gnomedateedit));
}


void
on_dateedit1_realize                   (GtkWidget       *widget,
                                        gpointer         user_data)
{
  smapi_outparm_t *result;
  dword tmp;
  struct tm time;
  assert(result=do_smapi(0x34,0x02,0,0,0,0,0,TRUE));

  /* time */
  tmp=result->dwParm4;
  time.tm_hour = byte_of_bcd8((tmp >> 16) & 0xff);
  time.tm_min = byte_of_bcd8((tmp >>  8) & 0xff);
  time.tm_sec = byte_of_bcd8((tmp >>  0) & 0xff);
  /* date */
  tmp=result->dwParm5;
  /* assume it's after 2000 */
  time.tm_year = 100 + byte_of_bcd8((tmp >> 16) & 0xff);
  time.tm_mon = byte_of_bcd8((tmp >>  8) & 0xff) -1;
  time.tm_mday = byte_of_bcd8((tmp >>  0) & 0xff);
  time.tm_isdst = -1; /* unavailable */
  gnome_date_edit_set_time(GNOME_DATE_EDIT(widget), mktime(&time));
}


void
on_daily_appointment_toggled           (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  appointment_changed(GTK_WIDGET(togglebutton));
}


void
on_daily_appointment_realize           (GtkWidget       *widget,
                                        gpointer         user_data)
{
  smapi_outparm_t *result;
  assert(result=do_smapi(0x34,0x02,0,0,0,0,0,TRUE));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget),
			       (result->dwParm5 & 0x80000000));
}

static int to_value(int history) {
  return minute_allowed[history];
}

static void timer_changed(int timer, GtkOptionMenu *optionmenu) {
  int value;
  
  value = to_value(gtk_option_menu_get_history(optionmenu));
  assert(do_smapi(0x32,0x03,timer | (value & 0x00ff),0,0,0,0,TRUE));
  /* high or auto synchronize. Seems unnecessary since high or auto can't be
     set */
}

void
on_battery_timer_changed               (GtkOptionMenu   *optionmenu,
                                        gpointer         user_data)
{
  int battery_option;

  if (setting_delay_timer)
    return;
  battery_option = gtk_option_menu_get_history
    (GTK_OPTION_MENU(lookup_widget(GTK_WIDGET(optionmenu), "optionmenu15")));
  timer_changed(to_timer(battery_option, TRUE), optionmenu);
}


void
on_ac_timer_changed                    (GtkOptionMenu   *optionmenu,
                                        gpointer         user_data)
{
  int ac_option;

  if (setting_delay_timer)
    return;
  ac_option = gtk_option_menu_get_history
    (GTK_OPTION_MENU(lookup_widget(GTK_WIDGET(optionmenu), "optionmenu16")));
  timer_changed(to_timer(ac_option, FALSE), optionmenu);

}


void
on_h_from_s_timer_changed              (GtkOptionMenu   *optionmenu,
                                        gpointer         user_data)
{
  int value;
  value = to_value(gtk_option_menu_get_history(optionmenu));
  assert(do_smapi(0x32,0x07, (value & 0x00ff),0,0,0,0,TRUE));
}

void
on_h_from_s_delay_timer_realize        (GtkWidget       *widget,
                                        gpointer         user_data)
{
  smapi_outparm_t *result;

  assert(result=do_smapi(0x32, 0x06, 0x00,0,0,0,0, TRUE));
  if (result->wParm1 & 0x0100) {
    /* capable */
    gtk_option_menu_set_history(GTK_OPTION_MENU(widget),
				num_to_option(result->wParm2 & 0x00ff));
  } else {
    /* not capable */
    gtk_widget_set_sensitive(widget, FALSE);
  }
}

static void handle_timer_control_realize(GtkWidget *widget, int bitnr) {
  smapi_outparm_t *result;
  int tmp;

  assert(result=do_smapi(0x22,0x02,0,0,0,0,0,FALSE));
  tmp=(result->wParm2 >> bitnr) & 0x0101;
  if (!(tmp & 0x0100)) {
    /* not capable */
    gtk_widget_set_sensitive(widget, FALSE);
  } else if (tmp & 0x0001) {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), TRUE);
  }
}

static void handle_timer_control_toggled(GtkToggleButton *togglebutton,
					 int bitnr) {
  smapi_outparm_t *result;
  int tmp;

  assert(result=do_smapi(0x22,0x02,0,0,0,0,0,FALSE));
  if (gtk_toggle_button_get_active(togglebutton)) {
      tmp=(result->wParm2 & 0x00ff) |  (0x0001 << bitnr);
  } else {
      tmp=(result->wParm2 & 0x00ff) & ~(0x0001 << bitnr);
  }
  assert(do_smapi(0x22,0x03,tmp,0,0,0,0,TRUE));
}

void
on_suspend_or_hibernate_checkbutton_realize
                                        (GtkWidget       *widget,
                                        gpointer         user_data)
{
  handle_timer_control_realize(widget, 0);
}


void
on_suspend_or_hibernate_checkbutton_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  handle_timer_control_toggled(togglebutton, 0);
}


void
on_stand_by_checkbutton_toggled        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  handle_timer_control_toggled(togglebutton, 1);
}


void
on_stand_by_checkbutton_realize        (GtkWidget       *widget,
                                        gpointer         user_data)
{
  handle_timer_control_realize(widget, 1);
}


void
on_lcd_off_toggled                     (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  handle_timer_control_toggled(togglebutton, 2);
}


void
on_lcd_off_realize                     (GtkWidget       *widget,
                                        gpointer         user_data)
{
  handle_timer_control_realize(widget, 2);
}


void
on_hdd_off_toggled                     (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  handle_timer_control_toggled(togglebutton, 3);
}


void
on_hdd_off_realize                     (GtkWidget       *widget,
                                        gpointer         user_data)
{
  handle_timer_control_realize(widget, 3);
}


void
on_Fn_label_realize                    (GtkWidget       *widget,
                                        gpointer         user_data)
{
  GdkColor color;
  gdk_color_parse ("blue", &color);
  gtk_widget_modify_fg (widget, GTK_STATE_NORMAL, &color);
}

void
on_pcmcia_suspend_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
  GConfClient *client;

  client = gconf_client_get_default();
  /* set the flag in gconf */
  gconf_client_set_bool(client, eject_pcmcia_before_suspend,
			gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button)),
			NULL);
}

void
on_pcmcia_suspend_realize              (GtkWidget       *widget,
                                        gpointer         user_data)
{
  GConfClient *client;

  if (geteuid() != 0) {
    gtk_widget_set_sensitive(widget, 0);
    return;
  }
  client = gconf_client_get_default();
  /* get the flag from gconf */
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget),
			       gconf_client_get_bool(client, eject_pcmcia_before_suspend, NULL));
}


void
on_pcmcia_hibernate_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
  GConfClient *client;

  client = gconf_client_get_default();
  /* set the flag in gconf */
  gconf_client_set_bool(client, eject_pcmcia_before_hibernate,
			gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button)),
			NULL);
}


void
on_pcmcia_hibernate_realize            (GtkWidget       *widget,
                                        gpointer         user_data)
{
  GConfClient *client;

  if (geteuid() != 0) {
    gtk_widget_set_sensitive(widget, 0);
    return;
  }
  client = gconf_client_get_default();
  /* get the flag from gconf */
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget),
			       gconf_client_get_bool(client, eject_pcmcia_before_hibernate, NULL));

}

void
on_closebutton6_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
  /* error in opening file */
  /* OR no such device error */
  gtk_main_quit();
}

gboolean
on_dialog7_delete_event                (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  /* error in opening file */
  /* OR no such device error */
  gtk_main_quit();
  return FALSE;
}

static smapidev_displaycap_t displaycapCMOS, displaycapCurr;

static void show_current_display_state
(smapidev_dispmode_t dispmode, GtkWidget *widget, int reverse_p) {
  int intRtn;
  smapidev_ablestate_t result;

  intRtn =
    smapidev_GetDisplayState
    (intFiledesc, SMAPIDEV_STATEPLACE_CURR, dispmode, &result);
  if ( intRtn ) {
    printf("%d\n", intRtn);
    return;
  }
  if (reverse_p) {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), !result);
  } else {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), result);
  }
}

static int initializing_display_state=FALSE;

static void set_display(int dispmode, GtkToggleButton *togglebutton,
			int reverse_p) {
  int intRtn, setting;

  if (initializing_display_state)
    return;
  if (gtk_toggle_button_get_active(togglebutton)) {
    if (reverse_p)
      setting = SMAPIDEV_ABLESTATE_DISABLED;
    else
      setting = SMAPIDEV_ABLESTATE_ENABLED;
  } else {
    if (reverse_p)
      setting = SMAPIDEV_ABLESTATE_ENABLED;
    else
      setting = SMAPIDEV_ABLESTATE_DISABLED;
  }
    
  /* I don't wish to change CMOS but only that seems to work. */
  intRtn = smapidev_SetDisplayState(
		intFiledesc,
		SMAPIDEV_STATEPLACE_CMOS_AND_CURR,
		dispmode, setting);
	if ( intRtn ) {
	  printf("%d\n", intRtn);
	  return;
	}
}

void
on_detect_crt_checkbutton_realize      (GtkWidget       *widget,
                                        gpointer         user_data)
{
  initializing_display_state = TRUE;
  show_current_display_state(SMAPIDEV_DISPMODE_CRT_DETECTION_IGNORE, widget,
			     TRUE);
  /* I fail to disable it */
  gtk_widget_set_sensitive(widget, FALSE);
  initializing_display_state = FALSE;
}

void
on_detect_crt_checkbutton_toggled      (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  set_display(SMAPIDEV_DISPMODE_CRT_DETECTION_IGNORE, togglebutton, TRUE);
}

void
on_enable_tv_checkbutton_realize       (GtkWidget       *widget,
                                        gpointer         user_data)
{
  /* I got displaycapCurr.tv == SMAPIDEV_DISPLAYCAPTV_NONE. Why? */
  show_current_display_state(SMAPIDEV_DISPMODE_TV, widget, FALSE);
  if (displaycapCurr.tv == SMAPIDEV_DISPLAYCAPTV_NONE) {
    gtk_widget_set_sensitive(widget, FALSE);
  }
}


void
on_enable_tv_checkbutton_toggled       (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  set_display(SMAPIDEV_DISPMODE_TV, togglebutton, FALSE);
}


void
on_select_tv_checkbutton_toggled       (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  set_display(SMAPIDEV_DISPMODE_SELECT_TV, togglebutton, FALSE);
}

void
on_select_tv_checkbutton_realize       (GtkWidget       *widget,
                                        gpointer         user_data)
{
  show_current_display_state(SMAPIDEV_DISPMODE_SELECT_TV, widget, FALSE);
  if (displaycapCurr.tv == SMAPIDEV_DISPLAYCAPTV_NONE) {
    gtk_widget_set_sensitive(widget, FALSE);
  }
}


void
on_display_frame_realize               (GtkWidget       *widget,
                                        gpointer         user_data)
{
  int intRtn;

	displaycapCMOS.sizeStruct = sizeof( displaycapCMOS );
	intRtn = smapidev_GetDisplayCapability(
		intFiledesc,
		SMAPIDEV_STATEPLACE_CMOS,
		&displaycapCMOS
	);
	if ( intRtn ) {
	  printf("%d\n", intRtn);
	  return;
	}
	displaycapCurr.sizeStruct = sizeof( displaycapCurr );
	intRtn = smapidev_GetDisplayCapability(
		intFiledesc,
		SMAPIDEV_STATEPLACE_CURR,
		&displaycapCurr
	);

	if ( intRtn ) {
	  printf("%d\n", intRtn);
	  return;
	}
}

void
on_internal_display_radiobutton_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  set_display(SMAPIDEV_DISPMODE_INTERNAL, togglebutton, FALSE);
}

void
on_internal_display_radiobutton_realize
                                        (GtkWidget       *widget,
                                        gpointer         user_data)
{
  show_current_display_state(SMAPIDEV_DISPMODE_INTERNAL, widget, FALSE);
}

void
on_external_display_radiobutton_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  set_display(SMAPIDEV_DISPMODE_CRT, togglebutton, FALSE);
}

void
on_external_display_radiobutton_realize
                                        (GtkWidget       *widget,
                                        gpointer         user_data)
{
  show_current_display_state(SMAPIDEV_DISPMODE_CRT, widget, FALSE);
}


void
on_dual_display_checkbutton_toggled    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  set_display(SMAPIDEV_DISPMODE_DUAL, togglebutton, FALSE);
}

void
on_dual_display_checkbutton_realize    (GtkWidget       *widget,
                                        gpointer         user_data)
{
  initializing_display_state = TRUE;
  show_current_display_state(SMAPIDEV_DISPMODE_DUAL, widget, FALSE);
  /* I fail to disable it */
  gtk_widget_set_sensitive(widget, FALSE);
  initializing_display_state = FALSE;
}

/*
 * This function returns the ioctl errno as a negative number 
 */
static int ioctl_rtcmosram( int intFiledesc, rtcmosram_ioparm_t *pioparmThe )
{
	int intRtn;

	intRtn = ioctl( intFiledesc, IOCTL_RTCMOSRAM_REQUEST, pioparmThe );
	if ( intRtn ) return -errno;
	return 0;
}

void
on_daylight_saving_time_checkbutton_realize
                                        (GtkWidget       *widget,
                                        gpointer         user_data)
{
	rtcmosram_ioparm_t ioparmMy;
	int intRtn;

	ioparmMy.in.wFunc = RTCMOSRAM_FUNC_DAYLIGHTSAVINGTIME_GET;
	intRtn = ioctl_rtcmosram( intFiledesc, &ioparmMy );
	if ( intRtn ) {
	  printf("%d\n", intRtn);
	  return;
	}
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget),
				     ioparmMy.out.dwParm1);
}

static void set_label_yes_no(GtkWidget *widget, char *label, int flag) {
  gtk_label_set_text(GTK_LABEL(lookup_widget(widget, label)),
		     flag ? "Yes" : "No");
}

static void set_hdd_label(GtkWidget *widget, char *label, int hdd_type) {
  char tmp_str[1024];

  if (hdd_type == 0) {
    gtk_label_set_text(GTK_LABEL(lookup_widget(widget, label)),
		       "Not present");
  } else {
    sprintf(tmp_str, "0x%x", hdd_type);
    gtk_label_set_text(GTK_LABEL(lookup_widget(widget, label)),
		       tmp_str);
  }
}

void
on_cmos_map                            (GtkWidget       *widget,
                                        gpointer         user_data)
{
  rtcmosram_ioparm_t ioparmMy;
  byte bHDD0Type, bHDD1Type, bHDD2Type, bHDD3Type;
  byte bDisplayOperatingMode;
  byte bDisketteDriveType;
  int intRtn;
  char tmp_str[1024];
  GtkLabel *tmp_label;

  ioparmMy.in.wFunc = RTCMOSRAM_FUNC_GETDATA;
  intRtn = ioctl_rtcmosram( intFiledesc, &ioparmMy );
  if ( intRtn ) {
    printf("%d\n", intRtn);
    return;
  }

  bDisketteDriveType = ioparmMy.data.bDisketteDriveType >> 4;

  bHDD0Type = ioparmMy.data.bHardDiskDrive01Type >> 4;
  bHDD1Type = ioparmMy.data.bHardDiskDrive01Type & 0xF;
  bHDD2Type = ioparmMy.data.bHardDiskDrive23Type >> 4;
  if ( bHDD2Type == 0xF ) bHDD2Type = ioparmMy.data.bHardDiskDrive2Type;
  bHDD3Type = ioparmMy.data.bHardDiskDrive23Type & 0xF;
  if ( bHDD3Type == 0xF ) bHDD3Type = ioparmMy.data.bHardDiskDrive3Type;

  bDisplayOperatingMode = (ioparmMy.data.bEquipment >> 4) & 3;

  set_label_yes_no(widget, "has_power_label", 
		   ioparmMy.data.bStatusD & 0x80);
  set_label_yes_no(widget, "has_lost_power_label",
		   ioparmMy.data.bStatusDiag & 0x80);
  set_label_yes_no(widget, "has_bad_checksum_label",
		   ioparmMy.data.bStatusDiag & 0x40);
  set_label_yes_no(widget, "has_bad_equipment_label",
		   ioparmMy.data.bStatusDiag & 0x20);
  set_label_yes_no(widget, "has_bad_memory_label",
		   ioparmMy.data.bStatusDiag & 0x10);
  set_label_yes_no(widget, "has_bad_time_label",
		   ioparmMy.data.bStatusDiag & 0x04);
  set_label_yes_no(widget, "failed_init_label",
		   ioparmMy.data.bStatusDiag & 0x08);

  sprintf(tmp_str, "0x%x", ioparmMy.data.bStatusShutdown);
  gtk_label_set_text(GTK_LABEL(lookup_widget(widget, "disk_drive_type_label")),
		     tmp_str);

  tmp_label = GTK_LABEL(lookup_widget(widget, "display_mode_label"));
  switch(bDisplayOperatingMode) {
  case 3: gtk_label_set_text(tmp_label, "Monochrome");
    break;
  case 2: 
    gtk_label_set_text(tmp_label, "80-column");
    break;
  case 1: 
    gtk_label_set_text(tmp_label, "40-column");
    break;
  default:
    gtk_label_set_text(tmp_label, "Reserved code");
  }

  set_label_yes_no(widget, "coprocessor_label",
		   ioparmMy.data.bEquipment & 2);

  tmp_label = GTK_LABEL(lookup_widget(widget, "disk_drive_type_label"));
  switch (bDisketteDriveType) {
  case 4:     gtk_label_set_text(tmp_label, "1.44 MB");
    break;
  case 6:     gtk_label_set_text(tmp_label, "2.88 MB");
    break;
  default: gtk_label_set_text(tmp_label, "Reserved code");
  }

  gtk_label_set_text(GTK_LABEL(lookup_widget(widget,
					     "disk_drive_num_label")),
		     ((ioparmMy.data.bEquipment >> 6) & 3) == 0 ? "1" : "?");

  set_label_yes_no(widget, "drive_present_label",
		   (ioparmMy.data.bEquipment & 1));

  set_hdd_label(widget, "drive0type_label", bHDD0Type);
  set_hdd_label(widget, "drive1type_label", bHDD1Type);
  set_hdd_label(widget, "drive2type_label", bHDD2Type);
  set_hdd_label(widget, "drive3type_label", bHDD3Type);

  sprintf(tmp_str, "%d KB", ioparmMy.data.bSizeBaseMemLow
	  + (word)(ioparmMy.data.bSizeBaseMemHigh << 8));
  gtk_label_set_text(GTK_LABEL(lookup_widget(widget,
					     "conventional_memory_label")),
		     tmp_str);

  sprintf(tmp_str, "%d KB", ioparmMy.data.bSizeExpMemLow +
	  (word)(ioparmMy.data.bSizeExpMemHigh << 8));
  gtk_label_set_text(GTK_LABEL(lookup_widget(widget,
					     "expansion_memory_label")),
		     tmp_str);

  sprintf(tmp_str, "%d KB", ioparmMy.data.wSizeUsableMem);
  gtk_label_set_text(GTK_LABEL(lookup_widget(widget,
					     "contiguous_memory_label")),
		     tmp_str);

}


void
on_display_map                         (GtkWidget       *widget,
                                        gpointer         user_data)
{
	smapidev_displayinfo_t displayinfoThe;
	int intRtn;
	int fMonDDC1, fMonDDC2;
  GtkLabel *tmp_label;

	displayinfoThe.sizeStruct = sizeof( displayinfoThe );
	intRtn = smapidev_GetDisplayInfo( intFiledesc, &displayinfoThe );
  if ( intRtn ) {
    printf("%d\n", intRtn);
    return;
  }

  tmp_label = GTK_LABEL(lookup_widget(widget, "panel_type_label"));
  switch ( displayinfoThe.bPanelType ) {
  case 0: gtk_label_set_text(tmp_label, "Monochrome STN LCD"); break;
  case 1: gtk_label_set_text(tmp_label, "Monochrome TFT LCD"); break;
  case 2: gtk_label_set_text(tmp_label, "Color STN LCD"); break;
  case 3: gtk_label_set_text(tmp_label, "Color TFT LCD"); break;
  default: gtk_label_set_text(tmp_label, "Unknown panel type");
  }
  
  tmp_label = GTK_LABEL(lookup_widget(widget, "panel_dimension_label"));
  switch ( displayinfoThe.bPanelDim ) {
  case 0: gtk_label_set_text(tmp_label, "640 x 480"); break;
  case 1: gtk_label_set_text(tmp_label, "800 x 600"); break;
  case 2: gtk_label_set_text(tmp_label, "1024 x 768"); break;
    /* TP560Z Tech Ref has "1024 x 1024" -- wrong! */
  case 3: gtk_label_set_text(tmp_label, "1280 x 1024"); break;
  case 4: gtk_label_set_text(tmp_label, "1408 x 1050"); break;
  default: gtk_label_set_text(tmp_label, "Unknown dimension");
  }

  tmp_label = GTK_LABEL(lookup_widget(widget, "monitor_type_label"));
  switch ( displayinfoThe.bCrtType ) {
  case 0x00: gtk_label_set_text(tmp_label, "No external display attached");
    break;
  case 0x10: gtk_label_set_text(tmp_label, "Color external display"); break;
  case 0x20: gtk_label_set_text(tmp_label, "Monochrome external display");
				break;
  case 0xFF: gtk_label_set_text(tmp_label, "Unknown monitor type"); break;
  default: gtk_label_set_text(tmp_label, "Unknown");
  }

  set_label_yes_no(widget, "ddc1_label", displayinfoThe.bCrtFeatures & 1);
  set_label_yes_no(widget, "ddc2_label", displayinfoThe.bCrtFeatures & 2);
}


void
on_processor_map                       (GtkWidget       *widget,
                                        gpointer         user_data)
{
	smapidev_cpuinfo_t cpuinfoThe;
	int intRtn;
  char tmp_str[1024];
  GtkLabel *tmp_label;

	cpuinfoThe.sizeStruct = sizeof( cpuinfoThe );
	intRtn = smapidev_GetCpuInfo( intFiledesc, &cpuinfoThe );
  if ( intRtn ) {
    printf("%d\n", intRtn);
    return;
  }

  tmp_label = GTK_LABEL(lookup_widget(widget, "processor_label"));
  switch ( cpuinfoThe.wManufacturer ) {
  case 1: gtk_label_set_text(tmp_label, "Intel"); break;
  case 2: gtk_label_set_text(tmp_label, "AMD"); break;
  default: gtk_label_set_text(tmp_label, "Unknown");
  }

  sprintf(tmp_str, "%d", cpuinfoThe.wType);
  gtk_label_set_text(GTK_LABEL(lookup_widget(widget,
					     "processor_type_label")),
		     tmp_str);

  sprintf(tmp_str, "%d", cpuinfoThe.wStepping);
  gtk_label_set_text(GTK_LABEL(lookup_widget(widget,
					     "stepping_label")),
		     tmp_str);
  
  sprintf(tmp_str, "%d MHz", cpuinfoThe.wClock);
  gtk_label_set_text(GTK_LABEL(lookup_widget(widget,
					     "cpu_clock_label")),
		     tmp_str);
  
  sprintf(tmp_str, "%d MHz", cpuinfoThe.wInternalClock);
  gtk_label_set_text(GTK_LABEL(lookup_widget(widget,
					     "internal_clock_label")),
		     tmp_str);
}

void
on_ultrabay_map                        (GtkWidget       *widget,
                                        gpointer         user_data)
{
  smapidev_ultrabayinfo_t ultrabayinfoThe;
  int intRtn;
  char *tmp_str;
	
	ultrabayinfoThe.sizeStruct = sizeof( ultrabayinfoThe );
	intRtn = smapidev_GetUltrabayInfo( intFiledesc, &ultrabayinfoThe );
  if ( intRtn ) {
    printf("%d\n", intRtn);
    return;
  }

  switch ( ultrabayinfoThe.bType ) {
  case 0x00: tmp_str = "Floppy Disk Drive";  break;
  case 0x01: tmp_str = "Serial Device"; break;
  case 0x02: tmp_str = "TV tuner"; break;
  case 0x10: tmp_str = "IDE"; break;
  case 0x20: tmp_str = "PCMCIA adapter"; break;
  case 0x30: tmp_str = "Battery"; break;
  case 0x40: tmp_str = "AC adapter"; break;
  case 0xFE: tmp_str = "No UltraBay exists"; break;
  default: tmp_str = "Unknown";
  }
  gtk_label_set_text(GTK_LABEL(lookup_widget(widget, "device_type_label")),
		     tmp_str);

  switch ( ultrabayinfoThe.bID ) {
  case 0x00: tmp_str = "Floppy Disk Drive"; break;
  case 0x01: tmp_str = "Cellular"; break;
  case 0x02: tmp_str = "TV Tuner"; break;
  case 0x10: tmp_str = "CD-ROM"; break;
  case 0x11: tmp_str = "IDE Hard Disk"; break;
  case 0x12: tmp_str = "DVD"; break;
  case 0x13: tmp_str = "ZIP Drive"; break;
  case 0x16: tmp_str = "DVD CD-RW"; break;
  default: tmp_str = "Unknown";
  }
  gtk_label_set_text(GTK_LABEL(lookup_widget(widget, "device_id_label")),
		     tmp_str);
}

/*
 * This function returns the ioctl errno as a negative number 
 */
static int ioctl_thinkpad( int intFiledesc, int intIoctlNum, unsigned long ulongIoctlArg )
{
	int intRtn;

	intRtn = ioctl( intFiledesc, intIoctlNum, ulongIoctlArg );
	if ( intRtn ) return -errno;
	return 0;
}

typedef struct _moduleinfo
{
	char	szVersion[LEN_VERSION_MAX+1];
} moduleinfo_t;

static int get_module_info(
	int intFiledesc,
	moduleinfo_t *pmoduleinfoThe
) {
	int intRtn;

	intRtn = ioctl_thinkpad(
		intFiledesc,
		IOCTL_THINKPAD_GETVER,
		(unsigned long)&(pmoduleinfoThe->szVersion)
	);
	if ( intRtn ) return intRtn;

	/*** Make sure that the returned string is terminated ***/
	pmoduleinfoThe->szVersion[LEN_VERSION_MAX] = '\0';

	return 0;
}

void
on_software_version_map                (GtkWidget       *widget,
                                        gpointer         user_data)
{
	smapidev_info_t infoThe;
	moduleinfo_t moduleinfoThe;
	int intRtn;

	intRtn = smapidev_GetInfo( &infoThe );
  if ( intRtn ) {
    printf("%d\n", intRtn);
    return;
  }

	intRtn = get_module_info( intFiledesc, &moduleinfoThe );
  if ( intRtn ) {
    printf("%d\n", intRtn);
    return;
  }

  gtk_label_set_text(GTK_LABEL(lookup_widget(widget, "thinkpad_module_label")),
		     moduleinfoThe.szVersion);

  gtk_label_set_text(GTK_LABEL(lookup_widget(widget, "smapidev_label")),
		     infoThe.szVersion);

}

void
on_app1_realize                        (GtkWidget       *widget,
                                        gpointer         user_data)
{
  GtkWidget *app;
  struct stat statBuf;

  if (stat("/proc/acpi", &statBuf) == 0) {
    app = create_dialog4();
    gtk_label_set_text(GTK_LABEL(lookup_widget(app, "label183")),
		       "Your system is using ACPI. Many power mangement functions in Configure-ThinkPad are disabled by ACPI. Please turn off APCI by adding \"acpi=off\" as a kernel boot parameter.");
    gtk_widget_show(app);
  }
}

