/***********************************************************************
 *
 * Author: Cheuksan Edward Wang
 *
 *     Copyright (c) 2003 Martin Fluch, All rights reserved
 *     
 *     This program is free software; you can redistribute it and/or 
 *     modify it under the terms of the GNU General Public License as 
 *     published by the Free Software Foundation; either version 2 of 
 *     the License, or (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *     GNU General Public License for more details.
 * 
 *     To receive a copy of the GNU General Public License, please write
 *     to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 *     Boston, MA 02111-1307 USA
 *
 ***********************************************************************/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "interface.h"
#include "support.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

/****** defines ******/

#define SZ_TPCTL_NAME "configure-thinkpad"

/****** variables ******/

static const char _szMyName[] = SZ_TPCTL_NAME;
static const char _szDefaultDeviceName[] = "/dev/thinkpad/thinkpad";
static const char _szDefaultDeviceName2[] = "/dev/thinkpad";

/*** flags for options ****/
/* They have to be ints, not flag_ts in order to work with getopt() */
int fdevice=0;

int intFiledesc;

GtkWidget *initialize()
{
  GtkWidget *app1;

	struct stat statBuf;
	int intOpenFlags;
	char *pchDeviceName;

	/*** Set up default ***/
	pchDeviceName = (char *)_szDefaultDeviceName;
	intOpenFlags = O_RDWR;

	if (stat(pchDeviceName,&statBuf)) {
		if (fdevice) {
			if (errno==ENOENT)
				printf( "%s: Device file %s does not exist. Exiting.\n", _szMyName, pchDeviceName );
			else
				printf( "%s: stat() of file %s failed. Exiting.\n", _szMyName, pchDeviceName );
			exit( EXIT_FAILURE );
		} else {
			if (stat(_szDefaultDeviceName2,&statBuf)) {
			  if (errno==ENOENT) {
			    app1 = create_dialog7();
			    gtk_label_set_text(GTK_LABEL(lookup_widget(app1,
								       "label205")),
					"Neither /dev/thinkpad/thinkpad nor /dev/thinkpad exists. Please create the device.");
			    gtk_window_set_title(GTK_WINDOW(app1), "No Device");
			    return app1;
			  } else
					printf( "%s: stat() of both %s and %s failed. Exiting.\n", _szMyName, pchDeviceName, _szDefaultDeviceName2 );
				exit( EXIT_FAILURE );
			} else {
				pchDeviceName = (char *)_szDefaultDeviceName2;
			}
		}
	}
	/*** Open the device ***/
	intFiledesc = open( pchDeviceName, intOpenFlags );
	if ( intFiledesc < 0 ) {
	  if ((errno == EBUSY) && (pchDeviceName == _szDefaultDeviceName2)) {
	      app1 = create_dialog7();
	  } else if ((errno == EACCES) && 
		     (pchDeviceName == _szDefaultDeviceName2)) {
	      app1 = create_dialog8();
	  } else if ((errno == EACCES) && 
		     (pchDeviceName == _szDefaultDeviceName)) {
	      app1 = create_dialog8();
	      gtk_label_set_text(GTK_LABEL(lookup_widget(app1, "label206")),
				 "You do not have read and write permissons on the ThinkPad device /dev/thinkpad/thinkpad. Please run this program as root or ask you administrator to give you permissions.");
	      return app1;
	  } else if (errno == ENODEV) {
	    /* no such device error */
	    app1 = create_dialog9();
	  } else {	    
		perror( SZ_TPCTL_NAME ": System error message is" );
		printf( "%s: Can't open device file %s with flags %s.  Exiting.\n", _szMyName, pchDeviceName, intOpenFlags == O_RDWR ? "O_RDWR" : (intOpenFlags == O_RDONLY ? "O_RDONLY" : "????" ) );
		exit( EXIT_FAILURE );
	  }
	/* what if it's readable */
	/* check permission denied in /dev/thinkpad */
	} else {
	  app1 = create_app1 ();
	}
	return app1;
}

int
main (int argc, char *argv[])
{
#ifdef ENABLE_NLS
  bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
  textdomain (GETTEXT_PACKAGE);
#endif

  gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE,
                      argc, argv,
                      GNOME_PARAM_APP_DATADIR, PACKAGE_DATA_DIR,
                      NULL);

	gtk_widget_show (initialize());
	  
	gtk_main ();
	return 0;
}

